$(function() {

    // hide action button
    $('#retry').removeClass('hide').hide();
    $('#next').removeClass('hide').hide();

    /**
     * ontouch event, addClass('active') for pressed-down effect
     */
    function onPress() {
        $(this).addClass('active');
    }

    function onUnpress() {
        $(this).removeClass('active');
    }
    $('.page-control a')
        .bind('touchstart', onPress)
        .bind('touchmove', onUnpress)
        .bind('touchend', onUnpress)
        .bind('touchcancel', onUnpress);

    /**
     * jazz app if has category value,
     * else open app detail window
     */
    function jazzOrOpenAppDetail(e) {
        e.preventDefault();
        var self = $(this);
        var uid = self.data('uid');
        var catId = self.data('cat-id');
        if (catId) {
            if (!self.hasClass('disabled')) {
                self.addClass('disabled');
                self.find('.grey6').text('...');
                Ti.App.fireEvent('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':jazz_app', {
                    uid: uid,
                    catId: catId,
                    groupType: GROUP_TYPE
                });
            }
        } else { // get app detail b4 open detail window
            var link = $('#link_' + uid);
            Ti.App.fireEvent('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':open_app_detail', {
                uid: uid,
                jazzed: (link.length == 0 || link.find('.jazzed').length != 0),
                jazzing: (link.find('.disabled').length != 0)
            });
        }
        return false;
    }

    /**
     * open another window for specific category
     */
    function openCategory(e) {
        e.preventDefault();
        var self = $(this);
        var categoryId = self.data('cid');
        var categoryName = self.text();
        Ti.App.fireEvent('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':open_category', {
            categoryId: categoryId,
            categoryName: categoryName,
        });
        return false;
    }

    /**
     * retry button 
     */
    $('#retry').click(function(e) {
        e.preventDefault();
        loadApps({
            mode: 'retry'
        });
        return false;
    });

    /**
     * load more (next page) button
     */
    $('#next').click(function(e) {
        e.preventDefault();
        loadApps({
            mode: 'next'
        });
        return false;
    });

    var waypointEnabled = false;
    var isLoading = false;
    /**
     * continuous scrolling (waypoint) 
     */
    function checkWaypoint() {
        if (waypointEnabled) {
            var offset = 100;
            var self = $(window);
            var viewportHeight = (window.innerHeight || self.height());
            var bodyHeight = $('body').height();
            var windowTop = self.scrollTop();
            if (bodyHeight <= (viewportHeight + windowTop + offset) && !isLoading) {
                loadApps({
                    mode: 'next'
                });
            }
        }
    }
    $(window).scroll(checkWaypoint);

    /**
     * create .share_bar html 
     * @param {Object} f - file data object
     */
    function createShareBar(f) {

        var html = '<div class="share_bar">';

        // jazz btn before jazz, has link. if already jazz, hide this link
        if (!f.jazzed) {
            html += '<div id="link_' + f.uid + '">';
            html += '<a class="floated_left" data-uid="' + f.uid + '" data-cat-id="' + f.catId + '">';
            html += '<i class="jazz_red_icon icon"></i>';
            html += '<strong class="grey6">Jazz</strong>'; // html += '<h4 class="grey6">Jazz</h4>';
            html += '</a>';
            html += '</div>';
        }

        // comment count
        if (f.comment > 0) {
            html += '<a data-uid="' + f.uid + '" class="floated_right ml10">';
            html += '<i class="comment_icon icon"></i>' + f.comment;
            html += '</a>';
        }

        // jazz count
        html += '<a data-uid="' + f.uid + '" class="floated_right">';
        html += '<i class="jazz_grey_icon icon"></i>';
        html += '<span id="count_' + f.uid + '">' + f.jazz + '</span>';
        html += '</a>';
        html += '</div>';

        return html;

    }

    /**
     * render app list
     *  
     * @param {Object} e
     *  - status {Boolean}
     *  - files {Array<Object>}
     *  - hasMore {Boolean}
     *  - pageNum {Number}
     *  - articleView {Boolean}
     */
    function renderApps(e) {
        Ti.API.info('[JMA] renderAppList, CONTENT_TYPE: ' + CONTENT_TYPE + ', GROUP_TYPE: ' + GROUP_TYPE + ', e.pageNum: ' + e.pageNum);
        Ti.API.info('[JMA] e.files = ' + JSON.stringify(e.files));

        if (e.status) {

            var html = '';

            for (var i = 0, len = e.files.length; i < len; i++) {

                var f = e.files[i];

                if (e.articleView) {

                    if (!f.caption.title) { // skip if title is null
                        continue;
                    }

                    // article view
                    html += '<div class="article relative clearfix mt25 pg' + e.pageNum + '">';
                    html += '<div class="tc-align floated_left article_img_container">';
                    html += '<a data-uid="' + f.uid + '">';
                    html += '<img src="' + f.preview + '" class="article_img"/>';
                    html += '</a>';
                    html += '</div>';
                    html += '<div class="content_area floated_left relative">';
                    html += '<h2><a data-uid="' + f.uid + '">' + f.caption.title + '</a></h2>';
                    html += '<p class="grey4 f12">';
                    html += '<i>' + f.caption.by + ', ' + f.caption.timeago + '</i>';
                    html += '</p>';
                    html += createShareBar(f);
                    html += '</div>';
                    html += '</div>';

                } else {

                    // gallery view
                    html += '<div class="app_card pg' + e.pageNum + '">';
                    html += '<div class="desc_header">';
                    html += '<a data-uid="' + f.uid + '">';
                    html += '<img class="thumb img' + e.pageNum + '" src="' + f.thumb + '"/>';
                    html += '</a>';
                    html += '<span class="desc ml10">';
                    html += '<strong class="desc_header_t"><a data-uid="' + f.uid + '">' + f.name + '</a></strong><br/>'; // html += '<h4 style="font-size:13px"><a data-uid="' + f.uid + '">' + f.name + '</a></h4><br/>';
                    if (CONTENT_TYPE == 'category' || !f.catId) { // unsupported apps dont hv catId
                        html += '<span class="grey5">' + f.catName + '</span><br/>';
                    } else {
                        html += '<span><a class="grey5" data-cid="' + f.catId + '">' + f.catName + '</a></span><br/>';
                    }
                    if (f.oldPrice === false) {
                        html += '<span class="grey5">' + f.price + '</span>';
                    } else {
                        html += '<span class="grey5 discount mr5">' + f.oldPrice + '</span>';
                        html += '<span class="grey6">' + f.price + '</span>';
                    }
                    html += '</span>';
                    html += '</div>';

                    // preview image
                    html += '<div class="prev_img">';
                    html += '<a data-uid="' + f.uid + '">';
                    html += '<img class="prev img' + e.pageNum + '" src="http://alicia.mobile9.com/download/tmb.php?src=' + f.preview + '"/>';
                    if (f.caption && f.caption.title) { // title maybe null
                        html += '<span class="tl-align">';
                        html += '<span class="prev_desc f12 grey4">';
                        html += '<span class="trunc">' + f.caption.title + '</span>';
                        html += '<i>' + f.caption.timeago + '</i>';
                        html += '</span>';
                        html += '</span>';
                    }
                    html += '</a>';
                    html += '</div>';
                    html += createShareBar(f);

                    // on-sale ribbon
                    if (f.onsale && !e.articleView) {
                        html += '<img src="./lib/ui/html/img/mdpi/ribbon_onsale.png" class="on_sale"/>';
                    }

                    html += '</div>';
                }

            }

            if (len == 0) { // blank slate
                html += '<div class="page_control">';
                html += '<p class="blank_slate">' + NO_ITEM + '</p>';
                html += '</div>';
            }

            $('#app_wrap').append(html);

            $('.pg' + e.pageNum + ' a[data-uid]').click(jazzOrOpenAppDetail);
            $('.pg' + e.pageNum + ' a[data-cid]').click(openCategory);

            $('.img' + e.pageNum).bind('error', function() {
                $(this).attr('src', './lib/ui/html/img/mdpi/grey_square.png');
            });

            if (e.hasMore) { // show load more / loading image
                $('#next').show();
                waypointEnabled = true;
                if (e.pageNum == 1) {
                    setTimeout(checkWaypoint, 1000);
                }
            } else {

            }

        } else { // show retry button
            $('#retry').show();
        }
        $('#load').hide();
        isLoading = false;

        Ti.API.info('[JMA] renderApps, waypointEnabled: ' + waypointEnabled + ', isLoading: ' + isLoading);

    }
    Ti.App.addEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':render_apps', renderApps);

    /**
     * show loading image
     * 
     * @param {Object} e
     *  - mode {String}
     */
    function loadApps(e) {
        Ti.API.info('[JMA] loadApps, e.mode: ' + e.mode + ', event = apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':load_apps');
        if (e.mode == 'init') {
            $('#app_wrap').html('');
        }
        $('#retry').hide();
        $('#next').hide();
        $('#load').show();
        isLoading = true;
        waypointEnabled = false;
        Ti.App.fireEvent('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':get_apps', {
            mode: e.mode
        });
    }
    Ti.App.addEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':load_apps', loadApps);

    /**
     * change jazz link to 'jazzed!'. called when finish jazz an app.
     * 
     * @param {Object} e
     *  - uid {String}
     *  - success {Boolean}
     *  - cancel {Boolean}
     *  - groupType {String}
     */
    function jazzDone(e) {
        Ti.API.info('[JMA] jazzDone, uid:' + e.uid + ', success:' + e.success + ', cancel:' + e.cancel + ', groupType:' + e.groupType);
        if (e.groupType != GROUP_TYPE) {
            return;
        }
        if (e.success) {
            var html = '<span class="floated_left jazzed">' + '<i class="jazz_red_icon icon"></i>' + '<h4>Jazzed!</h4>' + '</span>';
            $('#link_' + e.uid).html(html);
            var count = $('#count_' + e.uid);
            count.text(parseInt(count.text()) + 1);
        } else {
            var link = $('#link_' + e.uid + ' a[data-uid]');
            link.removeClass('disabled');
            link.find('.grey6').text('Jazz');
        }
    }
    Ti.App.addEventListener('jazz:' + CONTENT_TYPE + ':' + WIN_ID + ':jazz_done', jazzDone);

    /**
     * unload, remove any app-level event listener
     */
    function unload() {
        Ti.API.info('[JMA] apps.jsx unload');
        Ti.App.removeEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':unload', arguments.callee);
        Ti.App.removeEventListener('jazz:' + CONTENT_TYPE + ':' + WIN_ID + ':jazz_done', jazzDone);
        Ti.App.removeEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':render_apps', renderApps);
        Ti.App.removeEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':load_apps', loadApps);
    }
    Ti.App.addEventListener('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':unload', unload);

    Ti.API.info('[JMA] apps.jsx loaded');
    Ti.API.info('[JMA] apps.jsx fireEvent = apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':ready');
    Ti.App.fireEvent('apps:' + CONTENT_TYPE + ':' + WIN_ID + ':' + GROUP_TYPE + ':ready');

});