/**
 * js validation source url:
 * http://www.jazzmyapp.com/register/
 * 	?rd=http%3A%2F%2Fwww.jazzmyapp.com%2F
 * 	&nm=<FULL NAME>
 * 	&em=<EMAIL ADDR>
 * 	&rt=fb
 * 	&op=su
 * 	&grp=1 
 */
$(function() {

    var submitted = false;
    /**
     * submit sign up form 
     */
    $('#sign_up').submit(function(e) {
        e.preventDefault();
        if (validateInput()) { // no input error
            if (!submitted) {
                submitted = true;
                $('.input').prop('disabled', true);
                if (MODE == 'register') {
                    Ti.App.fireEvent('sign_up:register', {
                        em: $('#em').val(),
                        un: $('#un').val(),
                        nm: $('#nm').val(),
                        pw: $('#pw').val()
                    });
                } else if (MODE == 'update') {
                    Ti.App.fireEvent('sign_up:update_name', {
                        nm: $('#nm').val()
                    });
                }
            }
        }
        return false;
    });

    /**
     * @param {String} name
     * @param {String} value
     * @return {Boolean} 
     */
    function validateInput(name, value) {
        if (name) {
            switch (name) {
                case 'em':
                    if (!isValidEmail(value)) {
                        setError(name, INVALID_EMAIL);
                        return false;
                    }
                    break;
                case 'nm':
                    if (value.length < 3 || value.length > 255) {
                        setError(name, NAME_LENGTH);
                        return false;
                    }
                    break;
                case 'un':
                    if (value.length < 3 || value.length > 32) {
                        setError(name, USERNAME_LENGTH);
                        return false;
                    } else if (!isValidUsername(value)) {
                        setError(name, USERNAME_INVALID);
                        return false;
                    }
                    break;
                case 'pw':
                    if (value.length < 6 || value.length > 32) {
                        setError(name, PASSWORD_LENGTH);
                        return false;
                    }
                    break;
            }
            return true;
        } else {
            var i = 0,
                j = 0;
            $('.input').each(function() {
                i++;
                var self = $(this);
                if (validateInput(self.attr('id'), self.val())) {
                    j++;
                }
            });
            return i == j;
        }
    }

    /**
     * @param {String} email
     * @return {Boolean} 
     */
    function isValidEmail(email) {
        return /^([\w-\.]+@([\w-]+\.)+[\w-]{2,4})?$/.test(email);
    }

    /**
     * @param {String} username
     * @return {Boolean} 
     */
    function isValidUsername(username) {
        return /[\(\s)\[\]\&\"\'\,\<\>\~\`\!\@\#\$\^\ (\)\+\=\{\}\:\ ?\|\%\/\\]/.exec(username) == null;
    }

    /**
     * @param {String} id
     * @param {String} message 
     */
    function setError(id, message) {
        var el = $('#' + id);
        el.addClass('error-txtbox');
        el.siblings('.error_msg').text(message);
        el.siblings('.form_ps').addClass('hide');
        el.siblings('label').addClass('error-lbl');
    }

    /**
     * validate text input on blur event. 
     * remove error on key down event
     */
    $('.input')
        .blur(function() {
            var self = $(this);
            validateInput(self.attr('id'), self.val());
        })
        .keydown(function() {
            var self = $(this);
            self.removeClass('error-txtbox');
            self.siblings('.error_msg').text('');
            self.siblings('.form_ps').removeClass('hide');
            self.siblings('label').removeClass('error-lbl');
            $('#sysresp').html('');
        });

    /**
     * fired when sign up process has failed
     * reason might not set if got connection error
     * @param {Object} e
     *  - reason {Array<String>}
     */
    function sysResponse(e) {
        submitted = false;
        $('.input').prop('disabled', false);
        if (e.reason) {
            for (r in e.reason) {
                setError(r, e.reason[r]);
            }
            $('html, body').animate({
                scrollTop: 0
            }, 200);
        }
    }
    Ti.App.addEventListener('sign_up:sysresp', sysResponse);

    function unload() {
        Ti.App.removeEventListener('sign_up:sysresp', sysResponse);
    }
    window.onunload = unload;

});